# Called from intranet-earned-value-management-procs.tcl
#
# project_id ID of the main project to be tracked

set start_time [clock clicks -milliseconds]

if {"" eq $project_id} { set project_id 0 }
set main_project_id $project_id
if {"" eq $diagram_width} { set diagram_width 600 }
if {"" eq $diagram_height} { set diagram_height 400 }

# Create a random ID for the diagram
set diagram_id "project_eva_[expr round(rand() * 100000000.0)]"

# Get some basic information about the project and skip the diagram if the project doesn't exist.
db_0or1row project_info "
	select	to_char(coalesce(main_p.start_date::date - 1, now()::date - 1), 'J') as main_project_start_julian,
		to_char(coalesce(main_p.end_date::date + 1, now()::date + 1), 'J') as main_project_end_julian,

		(select	to_char(min(start_date), 'J') from im_projects p
		where	p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey)
		) as sub_project_start_julian,
		(select	to_char(max(end_date), 'J') from im_projects p
		where	p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey)
		) as sub_project_end_julian,

		(select	to_char(min(day::date), 'J') from im_projects p, im_hours h
		where	p.project_id = h.project_id and p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey)
		) as hours_start_julian,
		(select	to_char(max(day::date), 'J') from im_projects p, im_hours h
		where	p.project_id = h.project_id and p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey)
		) as hours_end_julian

	from	im_projects main_p
	where	main_p.project_id = :main_project_id
"
set show_diagram_p [info exists main_project_start_julian]


# ad_return_complaint 1 "[expr [clock clicks -milliseconds] - $start_time]"


# -----------------------------------------------------
# Calculate start and end date for the diagram
#
set diagram_start_julian $main_project_start_julian
if {"" ne $hours_start_julian && $hours_start_julian < $diagram_start_julian} { set diagram_start_julian $hours_start_julian }
if {"" ne $sub_project_start_julian && $sub_project_start_julian < $diagram_start_julian} { set diagram_start_julian $sub_project_start_julian }

set diagram_end_julian $main_project_end_julian
if {"" ne $hours_start_julian && $hours_end_julian > $diagram_end_julian} { set diagram_end_julian $hours_end_julian }
if {"" ne $sub_project_start_julian && $sub_project_end_julian > $diagram_end_julian} { set diagram_end_julian $sub_project_end_julian }

# ad_return_complaint 1 "<pre>\nmain_project_start_julian=$main_project_start_julian\nsub_project_start_julian=$sub_project_start_julian\nhours_start_julian=$hours_start_julian\ndiagram_start_julian=$diagram_start_julian\n</pre>"

# Don't show diagram if too short
if {[expr abs($diagram_end_julian - $diagram_start_julian)] < 3} { set show_diagram_p 0 }

# Handle the case that the interval is too long
if {[expr abs($diagram_end_julian - $diagram_start_julian)] > 400} {
    set diagram_start_julian $sub_project_start_julian
}

if {[expr abs($diagram_end_julian - $diagram_start_julian)] > 400} {
    set diagram_end_julian $sub_project_end_julian
}


# ad_return_complaint 1 "$diagram_end_julian - $diagram_start_julian"

# -----------------------------------------------------
# Create a hash to convert between julian and ISO dates
#
set julian_sql "
    	select	im_day_enumerator as day_date,
		to_char(im_day_enumerator, 'J') as day_julian
	from	im_day_enumerator(to_date(:diagram_start_julian, 'J'), to_date(:diagram_end_julian, 'J')+1)
"
db_foreach julian $julian_sql {
    set julian_hash($day_julian) $day_date
    set julian_hash($day_date) $day_julian
}

# ad_return_complaint 1 "<pre>diagram_start_julian=$diagram_start_julian\n diagram_end_julian=$diagram_end_julian\n[array get julian_hash]"

# -----------------------------------------------------
# Get resource assignments and planned units per task
#
set percentage_sql "
		select	parent.project_id as parent_project_id,
			to_char(parent.start_date, 'J') as parent_start_julian,
			to_char(parent.end_date, 'J') as parent_end_julian,
			child.project_id,
			to_char(child.start_date, 'J') as child_start_julian,
			to_char(child.end_date, 'J') as child_end_julian,
			t.planned_units as child_planned_units,
			u.user_id,
			coalesce(round(bom.percentage), 0) as percentage
		from	im_projects parent,
			im_projects child
			LEFT OUTER JOIN im_timesheet_tasks t ON (child.project_id = t.task_id)
			LEFT OUTER JOIN acs_rels r ON (child.project_id = r.object_id_one),
			im_biz_object_members bom,
			users u
		where	parent.project_id = :project_id and
			child.tree_sortkey between parent.tree_sortkey and tree_right(parent.tree_sortkey) and
			r.rel_id = bom.rel_id and
			bom.percentage is not null and bom.percentage > 0.0 and
			r.object_id_two = u.user_id and
			u.user_id not in (select member_id from group_distinct_member_map where group_id = [im_profile_skill_profile])
"

# ad_return_complaint 1 "<pre>[join [db_list_of_lists t $percentage_sql] "\n"]</pre>"
array set task_planned_units_hash {}
array set percent_hash {}
db_foreach percentages $percentage_sql {
    for {set j $child_start_julian} {$j <= $child_end_julian} {incr j} {
	set key "$j"

	# No assignments during the weekend
	array unset date_comps
	array set date_comps [util_memoize [list im_date_julian_to_components $j]]
	set dow $date_comps(day_of_week)

	# Sum up percentages, except for weekends
	if {6 ne $dow && 7 ne $dow} {
	    set perc 0.0
	    if {[info exists percent_hash($key)]} { set perc $percent_hash($key) }
	    set perc [expr $perc + $percentage]
	    set percent_hash($key) [expr round(100.0 * $perc) / 100.0]
	}
    }
}


# -----------------------------------------------------
# Get logged hours
#
set hours_sql "
		select	sum(h.hours) as hours,
			to_char(h.day::date, 'J') as day_julian
		from	im_hours h,
			im_projects main_p
		where	main_p.project_id = :main_project_id and
			h.project_id in (
				select	p.project_id
				from	im_projects p
				where	p.tree_sortkey between main_p.tree_sortkey and tree_right(main_p.tree_sortkey)     
			)
		group by h.day::date
		order by h.day::date
"

# ad_return_complaint 1 "<pre>[join [db_list_of_lists h $hours_sql] "\n"]</pre>"

array set logged_hours_hash {}
db_foreach hours $hours_sql {
    set key "$day_julian"
    set logged_hours_hash($key) $hours
}


# --------------------------------------------------------------
# Determine audit values
# --------------------------------------------------------------

# Which attributes should be stored?
# set attribute_list { percent_completed cost_bills_cache cost_delivery_notes_cache cost_expense_logged_cache cost_invoices_cache cost_purchase_orders_cache cost_quotes_cache cost_timesheet_logged_cache reported_hours_cache reported_days_cache}
set attribute_list { percent_completed reported_hours_cache}

set audit_sql "
	select	to_char(a.audit_date, 'J') as audit_date_julian,
		a.audit_value		
	from	im_audits a
	where	a.audit_object_id = :main_project_id and
		a.audit_date >= to_date(:diagram_start_julian, 'J') and
		a.audit_date <= to_date(:diagram_end_julian, 'J')
	order by a.audit_date
"
db_foreach audit_loop $audit_sql {
    set fields [split $audit_value "\n"]
    foreach field $fields {
	set key_val [split $field "\t"]
	set attribute_name [lindex $key_val 0]
	set attribute_value [lrange $key_val 1 end]

	if {[lsearch $attribute_list $attribute_name] < 0} { continue }

	set key "$audit_date_julian"
	set cmd "set ${attribute_name}_hash($key) \$attribute_value"
	eval $cmd
    }
}


# --------------------------------------------------------------
# Calculate the aggregated values
# --------------------------------------------------------------

set accumulated_percent 0.0
set max_percent_completed 0.0
set accumulated_logged_hours 0.0
for {set i $diagram_start_julian} {$i < $diagram_end_julian} {incr i} {
    set key "$i"

    set val 0
    if {[info exists percent_hash($key)]} { set val $percent_hash($key) }
    set accumulated_percent [expr $accumulated_percent + $val]
    set accumulated_percent_hash($key) $accumulated_percent

    set val 0
    if {[info exists logged_hours_hash($key)]} { set val $logged_hours_hash($key) }
    set accumulated_logged_hours [expr $accumulated_logged_hours + $val]
    set accumulated_logged_hours_hash($key) $accumulated_logged_hours

    if {[info exists percent_completed_hash($key)] && "" ne $percent_completed_hash($key) } { set max_percent_completed $percent_completed_hash($key) }
    set max_percent_completed_hash($key) $max_percent_completed
}

#ad_return_complaint 1 $accumulated_percent
if {$accumulated_percent == 0.0} { set show_diagram_p 0 }



# --------------------------------------------------------------
# Build the JSON data for the diagram stores
# --------------------------------------------------------------

# Initialize attribute values
foreach att $attribute_list { set $att 0.0 }

set step_width [expr round(($diagram_end_julian - $diagram_start_julian) / 20)]
if {$step_width < 1} { set step_width 1 }

set data_lines [list]
for {set i $diagram_start_julian} {$i < $diagram_end_julian} {set i [expr $i + $step_width]} {
    set key "$i"
    set iso_date $julian_hash($i)
    set data_line "{date: new Date('$iso_date')"
    set v 0.0
    if {0.0 != $accumulated_percent} { set v [expr 100.0 * $accumulated_percent_hash($key) / $accumulated_percent] }
    append data_line ", 'planned_work': $v"
    set v 0.0
    if {0.0 != $accumulated_logged_hours} { set v [expr 100.0 * $accumulated_logged_hours_hash($key) / $accumulated_logged_hours] }
    append data_line ", 'logged_hours': $v"
    append data_line ", 'percent_completed': $max_percent_completed_hash($key)"

    # Loop through all attributes and add attribute to the list of values
    foreach att $attribute_list {
	if {[info exists ${att}_hash($key)]} { 
	    # Write the new value to the attribute named variable
	    set v [expr "\$${att}_hash($key)"]
	    # Skip if the new values is "" for some reasons.
	    # This way, the value from the last iteration will be used,
	    # which makes for a smooth curve.
	    if {"" != $v} { set $att $v }
	}
	set v [expr "\$${att}"]
	append data_line ", '$att': $v"
    }

    append data_line "}"
    lappend data_lines $data_line
}

set data_json "\[\n"
append data_json [join $data_lines ",\n"]
append data_json "\n\]\n"

set fields_json "'planned_work', 'percent_completed'"


# --------------------------------------------------------------
# Build some JS auxillary fields
# --------------------------------------------------------------

set attributes_js ""
foreach att $attribute_list { 
    append attributes_js ", '$att'"
}


